#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
/* Illustrates binary file processing */
#define CODE_SIZE 11
typedef struct
{
    char code[CODE_SIZE]; // product code
    char name[61]; // product name
    char uom[4]; // unit of measure
    double amount;
    double ppu; // price per unit
} ProductT;
typedef struct
{
    char key[CODE_SIZE]; // sort key
    fpos_t pos; // position of product in file
} OrderT;
// shows the contents of a binary file holding ProductT records
void show_contents(char *fileName)
{
    FILE *pf;
    ProductT buf;
    int i;

    if (NULL ==(pf=fopen(fileName, "rb")))
    {
        perror(fileName);
        return;
    }
    printf("\nNo. %10s %60s %3s %s %s\n", "Code", "Product name", "UoM", "amount",  "price/unit");
    i=0;
    while (fread(&buf, sizeof(ProductT), 1, pf) > 0)
    {
        printf("%3d %10s %60s %3s %f %f\n", i, buf.code, buf.name, buf.uom, buf.amount, buf.ppu);
        i++;
    }
    fclose(pf);
}
//comparison function for qsort
int cmp(const void *s1, const void *s2)
{
    return(strcmp((char *)s1, (char *)s2));
}
// requested by problem 3.3
void sortFile(const char *inputFileName, const char *sortedFileName)
{
    struct stat statBuf;
    FILE *fpIn, *fpOut;
    ProductT buf;
    // find number of records in file
    if (-1 == stat(inputFileName, &statBuf))
    {
        perror(inputFileName);
        return;
    };
    int n = statBuf.st_size / sizeof(ProductT);
    // allocate space for order array
    OrderT *order = (OrderT *)calloc(sizeof(OrderT), n);
    // pen input file
    if (NULL == (fpIn = fopen(inputFileName, "rb")))
    {
        perror(inputFileName);
        return;
    }
    for (int i = 0; i < n; i++)
    {
        fgetpos(fpIn, &order[i].pos); // get position of record in original file
        fread(&buf, sizeof(ProductT), 1, fpIn); // read record
        strcpy(order[i].key, buf.code); // get key field value
    }
    qsort(order, n, sizeof(OrderT), cmp); // sort order array
    // open output file
    if (NULL == (fpOut = fopen(sortedFileName, "wb")))
    {
        perror(sortedFileName);
        return;
    }

    for (int i = 0; i < n; i++)
    {
        fsetpos(fpIn, &order[i].pos); // set position in input
        fread(&buf, sizeof(ProductT), 1, fpIn); // read record at position
        fwrite(&buf, sizeof(ProductT), 1, fpOut); // write product sequentially to sorted file
    }
    // close both files
    fclose(fpIn);
    fclose(fpOut);
}
// reads a product into a ProductT record pointed to by p
void readProduct(ProductT *p)
{
    int i;
    static char inbuf[128];
    static char *what[] = {"code", "name", "unit of measure", "amount", "price per unit", ""};
    for (i = 0; *what[i]; i++)
    {
        printf("%s=", what[i]);
        fgets(inbuf, sizeof(inbuf), stdin);
        switch(i)
        {
        case 0:
            sscanf(inbuf, "%[^\n]", p->code);
            break;
        case 1:
            sscanf(inbuf, "%[^\n]", p->name);
            break;
        case 2:
            sscanf(inbuf, "%[^\n]", p->uom);
            break;
        case 3:
            sscanf(inbuf, "%lf", &p->amount);
            break;
        case 4:
            sscanf(inbuf, "%lf", &p->ppu);
            break;
        default:
            break;
        }
    }
}

int main(int argc, char *argv[])
{
    FILE *pf;
    ProductT buf;
    int i, n;
    char *fileName="file.dat";
    if (argc > 1) fileName = argv[1];

    /*Create file */
    printf("\nInput the number of products, n=");
    scanf("%d%*c", &n);
    if (NULL ==(pf=fopen(fileName, "wb")))
    {
        perror(fileName);
        return 1;
    }
    for (i=1; i<=n; i++)
    {
        readProduct(&buf);
        fwrite(&buf, sizeof(ProductT), 1, pf);
    }
    fclose(pf);
    printf("\nFile contents:\n");
    show_contents(fileName);
    sortFile(fileName, "sortedFile.dat");
    printf("\nSorted file contents:\n");
    show_contents("sortedFile.dat");
    //getchar();
    return 0;
}
